/*                               -*- Mode: C -*- 
 * syscalls.c --- 
 * Author          : Denis Froschauer
 * Created On      : 
 * Last Modified By: David Grothe (dave@gcom.com)
 * RCS Id          : $Id: syscalls.c,v 1.4 1996/01/23 19:40:13 dave Exp $
 * ----------------______________________________________________
 *   Copyright (C) 1995  Denis Froschauer
 *   Copyright (C) 1997  David Grothe, Gcom, Inc <dave@gcom.com>
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Library General Public
 *    License as published by the Free Software Foundation; either
 *    version 2 of the License, or (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this library; if not, write to the Free
 *    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    You can reach me by email to
 *    100741.1151@compuserve.com
 */


#ident "@(#) LiS syscalls.c 2.14 3/22/02 16:39:00 "

#ifdef LINUX

#include <sys/stream.h>
#include <sys/stropts.h>	/* streams config symbols are here */
#include <sys/poll.h>		/* usr structs */
#include <linux/unistd.h>	/* kernel system call definitions */
#include <sys/osif.h>

/*
 * Do not evaluate the following if we are only doing
 * dependencies.
 */
#if !defined(DEP) && !defined(__NR_putpmsg)

# ifndef __NR_streams2
# error "unistd.h does not define __NR_putpmsg or __NR_streams2"
# else		/* Linus defined __NR_streams2 rather than __NR_putpmsg */
# define __NR_putpmsg	__NR_streams2
# endif

#endif

#define	NR_putpmsg_2_1	189	/* 2.1 and onward */


#if !defined(DEP) && !defined(__NR_getpmsg)

# ifndef __NR_streams1
# error "unistd.h does not define __NR_getpmsg or __NR_streams1"
# else		/* Linus defined __NR_streams1 rather than __NR_getpmsg */
# define __NR_getpmsg	__NR_streams1
# endif

#endif

#define	NR_getpmsg_2_1	188	/* 2.1 and onward */

#if 0
/*
 * In head.c
 */
extern int
lis_strputpmsg(struct inode *i, struct file *fp, 
	       void *ctlp, void *datp, int band, int flags) ;
extern int
lis_strgetpmsg(struct inode *i, struct file *fp, 
	       void *ctlp, void *datp, int *bandp, int *flagsp,
	       int doit) ;
#endif

/*
 * In poll.c
 */
#if defined(PORTABLE_POLL)
extern int	lis_poll(struct poll *fds, int nfds, int time_out) ;
#endif

/*
 * To lock or not to lock: only SMP kernels need to lock
 */
#if defined(KERNEL_2_1) && !defined(KERNEL_2_3) && defined(__SMP__)
#include <linux/smp_lock.h>
#define	Klock	lock_kernel()
#define	Kunlock	unlock_kernel()
#else
#define	Klock	
#define	Kunlock	
#endif

/*
 *  the following are for saving syscall entry points
 */
#ifdef __NR_spipe
static void *default_sys_pipe        = NULL;
#endif
static void *default_sys_putpmsg     = NULL;
static void *default_sys_getpmsg     = NULL;
#ifdef i386
static void *default_sys_putpmsg_2_1 = NULL;
static void *default_sys_getpmsg_2_1 = NULL;
#endif
#ifdef __NR_spipe
static void *default_sys_spipe       = NULL;
#endif
#ifdef __NR_fattach
static void *default_sys_fattach     = NULL;
#endif
#ifdef __NR_fdetach
static void *default_sys_fdetach     = NULL;
#endif

extern void     *sys_call_table[];

#ifdef __NR_spipe
/*
 *  these simplify notation when intercepting syscall entry points
 */
typedef asmlinkage int (*syscall_pipe_t)(unsigned long *);
#endif

/*  -------------------------------------------------------------------  */
/* sys_poll -  poll() syscall entry point
 *
 * STATUS: complete, tested
 */
#if defined(PORTABLE_POLL)
asmlinkage int
sys_poll(void *ubuff, ulong n, int tmout)
{
    return(lis_poll((struct poll *) ubuff, n, tmout)) ;

}/*sys_poll*/
#endif

/*  -------------------------------------------------------------------  */
#if defined(PRINT_TASK_ENABLED)
static void pfi(struct task_struct *tp, void *fldp, int fldv, char *name)
{
    char	*tp_char = (char *) tp ;
    char	*fldp_char = (char *) fldp ;

    if (fldp_char == NULL)
	fldp_char = tp_char ;

    printk("0x%03x: %s = %d %u 0x%x\n", fldp_char - tp_char, name,
    		fldv, fldv, fldv) ;
}

static void pfp(struct task_struct *tp, void *fldp, void *fldv, char *name)
{
    char	*tp_char = (char *) tp ;
    char	*fldp_char = (char *) fldp ;

    if (fldp_char == NULL)
	fldp_char = tp_char ;

    printk("0x%03x: %s = %p\n", fldp_char - tp_char, name, fldv) ;
}
#endif

void lis_print_task(struct task_struct *tp)
{
#if defined(PRINT_TASK_ENABLED)
    pfi(tp, &tp->state, tp->state, "state");
    pfi(tp, &tp->flags, tp->flags, "flags");
    pfi(tp, &tp->sigpending, tp->sigpending, "sigpending");
    pfp(tp, &tp->addr_limit, NULL, "addr_limit");
    pfp(tp, &tp->exec_domain, tp->exec_domain, "exec_domain");
    pfi(tp, &tp->need_resched, tp->need_resched, "need_resched");
    pfi(tp, &tp->counter, tp->counter, "counter");
#if !defined(KERNEL_2_3)
    pfi(tp, &tp->priority, tp->priority, "priority");
    pfi(tp, &tp->avg_slice, tp->avg_slice, "avg_slice");
    pfi(tp, &tp->last_processor, tp->last_processor, "last_processor");
    pfp(tp, &tp->next_run, tp->next_run, "next_run");
    pfp(tp, &tp->prev_run, tp->prev_run, "prev_run");
    pfp(tp, &tp->tarray_ptr, tp->tarray_ptr, "tarray_ptr");
    pfp(tp, &tp->tss, NULL, "tss");
    pfp(tp, &tp->signal, NULL, "signal");
    pfp(tp, &tp->sigqueue, tp->sigqueue, "sigqueue");
    pfp(tp, &tp->sigqueue_tail, tp->sigqueue_tail, "sigqueue_tail");
    pfi(tp, &tp->oom_kill_try, tp->oom_kill_try, "oom_kill_try");
    pfp(tp, &tp->wait_chldexit, tp->wait_chldexit, "wait_chldexit");
    pfi(tp, NULL, tp->dumpable, "dumpable");
    pfp(tp, &tp->vfork_sem, tp->vfork_sem, "vfork_sem");
#else
    pfp(tp, &tp->wait_chldexit, NULL, "wait_chldexit");
#endif
    pfi(tp, &tp->has_cpu, tp->has_cpu, "has_cpu");
    pfi(tp, &tp->processor, tp->processor, "processor");
    pfi(tp, &tp->lock_depth, tp->lock_depth, "lock_depth");
    pfp(tp, &tp->next_task, tp->next_task, "next_task");
    pfp(tp, &tp->prev_task, tp->prev_task, "prev_task");
    pfp(tp, &tp->binfmt, tp->binfmt, "binfmt");
    pfi(tp, &tp->exit_code, tp->exit_code, "exit_code");
    pfi(tp, &tp->exit_signal, tp->exit_signal, "exit_signal");
    pfi(tp, &tp->pdeath_signal, tp->pdeath_signal, "pdeath_signal");
    pfi(tp, &tp->personality, tp->personality, "personality");
    pfi(tp, NULL, tp->did_exec, "did_exec");
    pfi(tp, &tp->pid, tp->pid, "pid");
    pfi(tp, &tp->pgrp, tp->pgrp, "pgrp");
    pfi(tp, &tp->tty_old_pgrp, tp->tty_old_pgrp, "tty_old_pgrp");
    pfi(tp, &tp->session, tp->session, "session");
    pfi(tp, &tp->leader, tp->leader, "leader");
    pfp(tp, &tp->p_opptr, tp->p_opptr, "p_opptr");
    pfp(tp, &tp->p_pptr, tp->p_pptr, "p_pptr");
    pfp(tp, &tp->p_cptr, tp->p_cptr, "p_cptr");
    pfp(tp, &tp->p_ysptr, tp->p_ysptr, "p_ysptr");
    pfp(tp, &tp->p_osptr, tp->p_osptr, "p_osptr");
    pfp(tp, &tp->pidhash_next, tp->pidhash_next, "pidhash_next");
    pfp(tp, &tp->pidhash_pprev, tp->pidhash_pprev, "pidhash_pprev");
    pfi(tp, &tp->policy, tp->policy, "policy");
    pfi(tp, &tp->rt_priority, tp->rt_priority, "rt_priority");
    pfi(tp, &tp->it_real_value, tp->it_real_value, "it_real_value");
    pfi(tp, &tp->it_prof_value, tp->it_prof_value, "it_prof_value");
    pfi(tp, &tp->it_virt_value, tp->it_virt_value, "it_virt_value");
    pfi(tp, &tp->it_real_incr, tp->it_real_incr, "it_real_incr");
    pfi(tp, &tp->it_prof_incr, tp->it_prof_incr, "it_prof_incr");
    pfi(tp, &tp->it_virt_incr, tp->it_virt_incr, "it_virt_incr");
    pfp(tp, &tp->real_timer, NULL, "real_timer");
    pfp(tp, &tp->times, NULL, "times");
    pfi(tp, &tp->start_time, tp->start_time, "start_time");
    pfp(tp, &tp->per_cpu_utime, tp->per_cpu_utime, "per_cpu_utime");
    pfp(tp, &tp->per_cpu_stime, tp->per_cpu_stime, "per_cpu_stime");
    pfi(tp, &tp->min_flt, tp->min_flt, "min_flt");
    pfi(tp, &tp->maj_flt, tp->maj_flt, "maj_flt");
    pfi(tp, &tp->nswap, tp->nswap, "nswap");
    pfi(tp, &tp->cmin_flt, tp->cmin_flt, "cmin_flt");
    pfi(tp, &tp->cmaj_flt, tp->cmaj_flt, "cmaj_flt");
    pfi(tp, &tp->cnswap, tp->cnswap, "cnswap");
    pfi(tp, NULL, tp->swappable, "swappable");
    pfi(tp, &tp->uid, tp->uid, "uid");
    pfi(tp, &tp->euid, tp->euid, "euid");
    pfi(tp, &tp->suid, tp->suid, "suid");
    pfi(tp, &tp->fsuid, tp->fsuid, "fsuid");
    pfi(tp, &tp->gid, tp->gid, "gid");
    pfi(tp, &tp->egid, tp->egid, "egid");
    pfi(tp, &tp->sgid, tp->sgid, "sgid");
    pfi(tp, &tp->fsgid, tp->fsgid, "fsgid");
    pfi(tp, &tp->ngroups, tp->ngroups, "ngroups");
    pfp(tp, &tp->groups, tp->groups, "groups");
    pfi(tp, &tp->cap_effective, tp->cap_effective, "cap_effective");
    pfi(tp, &tp->cap_inheritable, tp->cap_inheritable, "cap_inheritable");
    pfi(tp, &tp->cap_permitted, tp->cap_permitted, "cap_permitted");
    pfp(tp, &tp->user, tp->user, "user");
    pfp(tp, &tp->rlim, tp->rlim, "rlim");
    pfi(tp, &tp->used_math, tp->used_math, "used_math");
    pfp(tp, &tp->comm, tp->comm, "comm");
    pfi(tp, &tp->link_count, tp->link_count, "link_count");
    pfp(tp, &tp->tty, tp->tty, "tty");
    pfp(tp, &tp->semundo, tp->semundo, "semundo");
    pfp(tp, &tp->semsleeping, tp->semsleeping, "semsleeping");
    pfp(tp, &tp->fs, tp->fs, "fs");
    pfp(tp, &tp->files, tp->files, "files");
    pfp(tp, &tp->mm, tp->mm, "mm");
    pfp(tp, &tp->sigmask_lock, NULL, "sigmask_lock");
    pfp(tp, &tp->sig, tp->sig, "sig");
    pfp(tp, &tp->blocked, NULL, "blocked");
    pfi(tp, &tp->sas_ss_sp, tp->sas_ss_sp, "sas_ss_sp");
    pfi(tp, &tp->sas_ss_size, tp->sas_ss_size, "sas_ss_size");
    pfi(tp, &tp->parent_exec_id, tp->parent_exec_id, "parent_exec_id");
    pfi(tp, &tp->self_exec_id, tp->self_exec_id, "self_exec_id");
#else
    printk("lis_print_task: not compiled with PRINT_TASK_ENABLED defined\n");
#endif
}

/*  -------------------------------------------------------------------  */
/* sys_putpmsg -  putpmsg syscall entry point
 *
 * STATUS: complete, tested
 */
asmlinkage int
sys_putpmsg(int fd,
	    struct strbuf *ctl,
	    struct strbuf *dat,
	    int band,
	    int flags)
{
    struct file		* file;
    struct inode	* inode = NULL;
    int			  ret ;

    /*
     * The following checks are patterned after sys_write
     */
    Klock ;
#if defined(KERNEL_2_1)
    if (   (file = fget(fd)) == NULL
	|| file->f_dentry == NULL
	|| (inode = file->f_dentry->d_inode) == NULL
       )
    {
	if (file != NULL)
	    fput(file) ;
#if defined(PUTPMSG_DEBUG)
	printk("sys_putpmsg: bad file --\n") ;
	printk("sys_putpmsg: fd=%d\n", fd) ;
	printk("sys_putpmsg: ctl=%p\n", ctl) ;
	printk("sys_putpmsg: dat=%p\n", dat) ;
	printk("sys_putpmsg: band=%d\n", band) ;
	printk("sys_putpmsg: flags=%x\n", flags) ;
	printk("sys_putpmsg: file=%p\n", file) ;
	if (file != NULL)
	{
	    printk("sys_putpmsg: file->f_dentry=%p\n", file->f_dentry) ;
	    printk("sys_putpmsg: inode=%p\n", inode) ;
	}
	printk("sys_putpmsg: current=%p\n", current) ;
	printk("sys_putpmsg: current->comm=%s\n", current->comm) ;
	lis_print_task(current) ;
#endif
	Kunlock ;
	return -EBADF;
    }
#else
    if (fd>=NR_OPEN || !(file=current->files->fd[fd]) || !(inode=file->f_inode))
    {
	Kunlock ;
	return -EBADF;
    }
#endif
    if (!(file->f_mode & 2))
    {
#if defined(KERNEL_2_1)
	fput(file) ;
#endif
	Kunlock ;
	return -EBADF;
    }
    if (!file->f_op || !file->f_op->write)
    {
#if defined(KERNEL_2_1)
	fput(file) ;
#endif
	Kunlock ;
	return -EINVAL;
    }

    LOCK_INO(inode);
    ret = lis_strputpmsg(inode,file,ctl,dat,band,flags);
    ULOCK_INO(inode);

#if defined(KERNEL_2_1)
    fput(file) ;
#endif
    Kunlock ;
    return(ret) ;

}/*sys_putpmsg*/

/*  -------------------------------------------------------------------  */
/* sys_getpmsg -  getpmsg syscall entry point
 *
 * STATUS: complete, tested
 */
asmlinkage int
sys_getpmsg(int fd,
	    struct strbuf *ctl,
	    struct strbuf *dat,
	    int *bandp,
	    int *flagsp)
{
    struct file		* file;
    struct inode	* inode;
    int			  ret ;

    /*
     * The following checks are patterned after sys_read
     */
    Klock ;
#if defined(KERNEL_2_1)
    if (   (file = fget(fd)) == NULL
	|| file->f_dentry == NULL
	|| (inode = file->f_dentry->d_inode) == NULL
       )
    {
	if (file != NULL)
	    fput(file) ;

	Kunlock ;
	return -EBADF;
    }
#else
    if (fd>=NR_OPEN || !(file=current->files->fd[fd]) || !(inode=file->f_inode))
    {
	Kunlock ;
	return -EBADF;
    }
#endif
    if (!(file->f_mode & 1))
    {
#if defined(KERNEL_2_1)
	fput(file) ;
#endif
	Kunlock ;
	return -EBADF;
    }
    if (!file->f_op || !file->f_op->read)
    {
#if defined(KERNEL_2_1)
	fput(file) ;
#endif
	Kunlock ;
	return -EINVAL;
    }

    LOCK_INO(inode);
    ret = lis_strgetpmsg(inode,file,ctl,dat,bandp,flagsp,1);
    ULOCK_INO(inode);

#if defined(KERNEL_2_1)
    fput(file) ;
#endif
    Kunlock ;
    return(ret) ;

}/*sys_getpmsg*/

/*
 *  pipe() system call wrapper
 */
asmlinkage int
lis_sys_pipe( unsigned int *fildes )
{
    unsigned int fd[2];
    int error;

    Klock;
    error = lis_pipe(fd);
    Kunlock;

    if (!error) {
	if (copy_to_user(fildes, fd, 2*sizeof(unsigned int)))
	    error = -EFAULT;
    }
    return error;
}

/*
 *  fattach() system call wrapper
 */
asmlinkage int
lis_sys_fattach( int fd, const char *path )
{
    struct file *f;
    int error;

    Klock;
    error = -EBADF;
    if ((f = fcheck(fd)))
	error = lis_fattach( f, path );
    Kunlock;

    return error;
}

/*
 *  fdetach() system call wrapper
 */
asmlinkage int
lis_sys_fdetach( const char *path )
{
    char *tmp = getname(path);
    int error = PTR_ERR(tmp);
    extern int lis_fdetach_all(void) ;

    if (!IS_ERR(tmp)) {
	Klock;
	if (strcmp( tmp, "*" ) == 0)
	    error = lis_fdetach_all();
	else
	    error = lis_fdetach(path);  /* arg must be in user space */
	Kunlock;
	putname(tmp);
    }

    return error;
}

void
hook_syscalls(void)
{
#ifdef __NR_spipe
    /*
     *  get some real entry points, for testing/passing through
     */
    default_sys_pipe  = (void *) sys_call_table[__NR_pipe];
#endif
    /*
     *  save the old entry points
     */
    default_sys_putpmsg = (void *) sys_call_table[__NR_putpmsg];
    default_sys_getpmsg = (void *) sys_call_table[__NR_getpmsg];
#ifdef i386
    default_sys_putpmsg_2_1 = (void *) sys_call_table[NR_putpmsg_2_1];
    default_sys_getpmsg_2_1 = (void *) sys_call_table[NR_getpmsg_2_1];
#endif
#ifdef PORTABLE_POLL
    default_sys_poll    = (void *) sys_call_table[__NR_poll];
#endif
#ifdef __NR_spipe
    default_sys_spipe   = (void *) sys_call_table[__NR_spipe];
#endif
#ifdef __NR_fattach
    default_sys_fattach = (void *) sys_call_table[__NR_fattach];
#endif
#ifdef __NR_fdetach
    default_sys_fdetach = (void *) sys_call_table[__NR_fdetach];
#endif

        /* 
     *  now hook our entry points
        */
    sys_call_table[__NR_putpmsg]	= (void *)sys_putpmsg;
    sys_call_table[__NR_getpmsg]	= (void *)sys_getpmsg;
#ifdef i386
        sys_call_table[  NR_putpmsg_2_1]	= (void *)sys_putpmsg;
        sys_call_table[  NR_getpmsg_2_1]	= (void *)sys_getpmsg;
#endif
#if defined(PORTABLE_POLL)
        sys_call_table[__NR_poll]		= (void *)sys_poll;
#endif
#ifdef __NR_spipe
    sys_call_table[__NR_spipe]          = (void *)lis_sys_pipe;
#endif
#ifdef __NR_fattach
    sys_call_table[__NR_fattach]        = (void *)lis_sys_fattach;
#endif
#ifdef __NR_fdetach
    sys_call_table[__NR_fdetach]        = (void *)lis_sys_fdetach;
#endif

        return;
}

void
restore_syscalls(void)
{
    sys_call_table[__NR_putpmsg]     = (void *)default_sys_putpmsg;
    sys_call_table[__NR_getpmsg]     = (void *)default_sys_getpmsg;
#ifdef i386
    sys_call_table[  NR_putpmsg_2_1] = (void *)default_sys_putpmsg_2_1;
    sys_call_table[  NR_getpmsg_2_1] = (void *)default_sys_getpmsg_2_1;
#endif
#if defined(PORTABLE_POLL)
    sys_call_table[__NR_poll]	     = (void *)default_sys_poll;
#endif
#ifdef __NR_spipe
    sys_call_table[__NR_spipe]       = (void *) default_sys_spipe;
#endif
#ifdef __NR_fattach
    sys_call_table[__NR_fattach]     = (void *) default_sys_fattach;
#endif
#ifdef __NR_fdetach
    sys_call_table[__NR_fdetach]     = (void *) default_sys_fdetach;
#endif

        return;
}

#endif				/* LINUX */
