#!/bin/sh

if [ -f config.in ]; then
. config.in
fi

if [ "$KSRC..." = "..." ]; then
    KSRC=${1:-/usr/src/linux}
fi

DIR=`pwd`

ARCH=`uname -m | sed -e s/i.86/i386/ -e s/sun4u/sparc64/ -e 's/arm.*/arm/' -e s/sa110/arm/`

if ! grep -s -q __NR_putpmsg $KSRC/include/asm/unistd.h &&
   ! grep -s -q __NR_streams $KSRC/include/asm/unistd.h
then
    PRESENT=0
else
    PRESENT=1
fi

#
# Figure out kernel version
# Return the version string in KVER (e.g. 2.2.6)
#
get_kver()
{
    if [ -f $KSRC/include/linux/version.h ]; then
	cc -E -dM -include $KSRC/include/linux/version.h dummy.c >kver
	KVER=`grep UTS_RELEASE kver | \
	     sed -e 's/^.*UTS_RELEASE.//' -e's/"//g'`
    elif [ -f $KSRC/vmlinux ]; then
	KVDEC=`nm -g $KSRC/vmlinux | grep Version_ | sed -e 's/^.*_//'`
	KMAJ=`expr $KVDEC / 65536`
	KMIN=`expr $KVDEC / 256`
	KMIN=`expr $KMIN % 256`
	KPCH=`expr $KVDEC % 256`
	KVER="$KMAJ.$KMIN.$KPCH"
    elif grep -q "Red Hat" /etc/issue && rh_get_system_map; then
	KVDEC=`grep Version_ $SMAP | sed -e 's/^.*_//'`
	KMAJ=`expr $KVDEC / 65536`
	KMIN=`expr $KVDEC / 256`
	KMIN=`expr $KMIN % 256`
	KPCH=`expr $KVDEC % 256`
	KVER="$KMAJ.$KMIN.$KPCH"
    else
        echo "The file $KSRC/include/linux/version.h is not present."
	KVER="xxx.xxx"
    fi
}

if [ "$KVER..." = "..." ]; then
    get_kver				# not set, try to figure it out
fi
vers=$KVER

case $vers in
    2.0.*)
	if [ "$ARCH" != "i386" ]; then
	    echo "You cannot install LiS for $vers kernel on this architecture."
	    exit 1
	fi
	v=`echo $vers | sed -e 's/2\.0\.//'`
	if   [ $v -ge 36 ]; then		# 36 and up
	    if [ $PRESENT -eq 0 ]; then
		echo "STREAMS hooks should be standard in kernel version $vers"
		echo "but they are not present in your kernel."
	    fi
	    if ! grep -s -q exit_files $KSRC/kernel/ksyms.c; then
		exit 0		# patch not installed
	    fi
	    cd $KSRC
	    patch -p1 <$DIR/patches_rm.ksyms.36
	    exit $?
	elif [ $v -eq 35 ]; then		# 35
	    if [ $PRESENT -ne 0 ]; then
		cd $KSRC
		patch -p1 <$DIR/patches_rm.hooks
		exit $?
	    else
		exit 0
	    fi
	elif [ $v -ge 28 ]; then		# 28 - 34
	    if [ $PRESENT -ne 0 ]; then
	        cd $KSRC
	        patch -p1 -R <$DIR/patches.kernel.28-34
	        exit $?
	    else
		exit 0
	    fi
	else					# 27 or earlier
	    echo "Cannot remove STREAMS hooks from kernel version $vers"
	    echo "Must be at least kernel version 2.0.28.  Continuing..."
	    exit 0
	fi
	;;
    2.1.*)
	v=`echo $vers | sed -e 's/2\.0\.//'`
	if [ $v -ge 115 ]; then
	    if [ $PRESENT -eq 0 ]; then
		echo "STREAMS hooks should be standard in kernel version $vers"
		echo "but they are not present in your kernel."
	    fi
	fi
	exit 0
	;;
    2.2.*|2.3.*|2.4.*)
	if [ $PRESENT -eq 0 ]; then
	    echo "STREAMS hooks should be standard in kernel version $vers"
	    echo "but they are not present in your kernel."
	fi
	exit 0
	;;
    *)
	echo "cannot determine linux version from version.h"
	exit 1
	;;
esac
